# -*- coding: utf-8; -*-
################################################################################
#
#  wuttaweb -- Web App for Wutta Framework
#  Copyright © 2024-2025 Lance Edgar
#
#  This file is part of Wutta Framework.
#
#  Wutta Framework is free software: you can redistribute it and/or modify it
#  under the terms of the GNU General Public License as published by the Free
#  Software Foundation, either version 3 of the License, or (at your option) any
#  later version.
#
#  Wutta Framework is distributed in the hope that it will be useful, but
#  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
#  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
#  more details.
#
#  You should have received a copy of the GNU General Public License along with
#  Wutta Framework.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
Essential views for convenient includes

Most apps should include this module::

   pyramid_config.include("wuttaweb.views.essential")

That will in turn include the following modules:

* :mod:`wuttaweb.views.common`
* :mod:`wuttaweb.views.auth`
* :mod:`wuttaweb.views.email`
* :mod:`wuttaweb.views.settings`
* :mod:`wuttaweb.views.progress`
* :mod:`wuttaweb.views.people`
* :mod:`wuttaweb.views.roles`
* :mod:`wuttaweb.views.users`
* :mod:`wuttaweb.views.upgrades`
* :mod:`wuttaweb.views.tables`
* :mod:`wuttaweb.views.alembic`
* :mod:`wuttaweb.views.views`

You can also selectively override some modules while keeping most
defaults.

That uses a slightly different approach.  First here is how to do it
while keeping all defaults::

   from wuttaweb.views import essential

   essential.defaults(pyramid_config)

To override, specify the default module with your preferred alias like
so::

   essential.defaults(pyramid_config, **{
       "wuttaweb.views.users": "poser.web.views.users",
       "wuttaweb.views.upgrades": "poser.web.views.upgrades",
   })
"""


def defaults(config, **kwargs):  # pylint: disable=missing-function-docstring

    def mod(spec):
        return kwargs.get(spec, spec)

    config.include(mod("wuttaweb.views.common"))
    config.include(mod("wuttaweb.views.auth"))
    config.include(mod("wuttaweb.views.email"))
    config.include(mod("wuttaweb.views.settings"))
    config.include(mod("wuttaweb.views.progress"))
    config.include(mod("wuttaweb.views.people"))
    config.include(mod("wuttaweb.views.roles"))
    config.include(mod("wuttaweb.views.users"))
    config.include(mod("wuttaweb.views.upgrades"))
    config.include(mod("wuttaweb.views.tables"))
    config.include(mod("wuttaweb.views.alembic"))
    config.include(mod("wuttaweb.views.views"))


def includeme(config):  # pylint: disable=missing-function-docstring
    defaults(config)



# -*- coding: utf-8; -*-

import sys
from unittest.mock import patch

import colander
from pyramid.httpexceptions import HTTPNotFound

from wuttaweb.views import settings as mod
from wuttaweb.testing import WebTestCase


class TestAppInfoView(WebTestCase):

    def setUp(self):
        self.setup_web()
        self.pyramid_config.include("wuttaweb.views.essential")

    def make_view(self):
        return mod.AppInfoView(self.request)

    def test_get_grid_data(self):
        view = self.make_view()

        # empty data by default
        data = view.get_grid_data()
        self.assertEqual(data, [])

        # 'partial' request returns data
        self.request.GET = {"partial": "1"}
        data = view.get_grid_data()
        self.assertIsInstance(data, list)
        self.assertTrue(data)

    def test_index(self):
        # sanity/coverage check
        view = self.make_view()
        response = view.index()

    def test_configure_get_simple_settings(self):
        # sanity/coverage check
        view = self.make_view()
        simple = view.configure_get_simple_settings()

    def test_configure_get_context(self):
        # sanity/coverage check
        view = self.make_view()
        context = view.configure_get_context()

    def test_configure_check_timezone(self):
        view = self.make_view()

        # normal
        with patch.object(self.request, "GET", new={"tzname": "America/Chicago"}):
            result = view.configure_check_timezone()
            self.assertFalse(result["invalid"])

        # invalid
        with patch.object(self.request, "GET", new={"tzname": "bad_name"}):
            result = view.configure_check_timezone()
            # nb. this check won't work for python 3.8
            if sys.version_info >= (3, 9):
                self.assertEqual(
                    result["invalid"], "'No time zone found with key bad_name'"
                )

        # missing input
        with patch.object(self.request, "GET", new={}):
            result = view.configure_check_timezone()
            self.assertEqual(result["invalid"], "Must provide 'tzname' parameter.")


class TestSettingView(WebTestCase):

    def make_view(self):
        return mod.SettingView(self.request)

    def test_get_grid_data(self):

        # empty data by default
        view = self.make_view()
        query = view.get_grid_data(session=self.session)
        data = query.all()
        self.assertEqual(len(data), 0)

        # unless we save some settings
        self.app.save_setting(self.session, "foo", "bar")
        self.session.commit()
        query = view.get_grid_data(session=self.session)
        data = query.all()
        self.assertEqual(len(data), 1)

    def test_configure_grid(self):
        model = self.app.model
        view = self.make_view()
        grid = view.make_grid(model_class=model.Setting)
        self.assertFalse(grid.is_linked("name"))
        view.configure_grid(grid)
        self.assertTrue(grid.is_linked("name"))

    def test_configure_form(self):
        view = self.make_view()
        form = view.make_form(fields=view.get_form_fields())
        self.assertNotIn("value", form.required_fields)
        view.configure_form(form)
        self.assertIn("value", form.required_fields)
        self.assertFalse(form.required_fields["value"])

    def test_unique_name(self):
        model = self.app.model
        view = self.make_view()

        setting = model.Setting(name="foo")
        self.session.add(setting)
        self.session.commit()

        with patch.object(view, "Session", return_value=self.session):

            # invalid if same name in data
            node = colander.SchemaNode(colander.String(), name="name")
            self.assertRaises(colander.Invalid, view.unique_name, node, "foo")

            # but not if name belongs to current setting
            view.editing = True
            self.request.matchdict = {"name": "foo"}
            node = colander.SchemaNode(colander.String(), name="name")
            self.assertIsNone(view.unique_name(node, "foo"))
